class KTDKError(Exception):
    def __init__(self, msg: str = None, **kwargs):
        super().__init__()
        self.msg = msg

    def __str__(self):
        return f"{self.__class__.__name__}: {self.msg}"


class KTDKAssertionError(KTDKError):
    pass


class RequireFailedError(KTDKAssertionError):
    pass


class KillCheckError(KTDKAssertionError):
    pass


class RequiredTaskFailed(KTDKAssertionError):
    pass


class TaskRunFailed(KTDKError):
    pass


class ContextIsNotInitializedError(KTDKError):
    def __init__(self, class_name: str):
        message = f'Context has not been initialized yet for class: {class_name}'
        super(ContextIsNotInitializedError, self).__init__(msg=message)



import difflib
from typing import AnyStr

from ktdk import utils


def get_context_diff(expected: AnyStr, provided: AnyStr, keep_ends=True):
    def _split_string(orig):
        if isinstance(orig, str):
            orig = orig.splitlines(keepends=keep_ends)
        return orig

    decoded_prov = utils.try_decode(provided)
    decoded_exp = utils.try_decode(expected)
    if decoded_exp and decoded_prov:
        expected = _split_string(decoded_exp)
        provided = _split_string(decoded_prov)

    return difflib.context_diff(expected, provided, fromfile='expected', tofile='provided')
